import argparse
import subprocess
from loguru import logger
import sys
import shutil

from concat_stories.snapchat_dl import SnapchatDL
from concat_stories.con_stories import ConcatStories


def is_ffmpeg_installed():
  try:
    result = subprocess.run(["ffmpeg", "-version"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    return result.returncode == 0
  except FileNotFoundError:
    return False


def main():
  parser = argparse.ArgumentParser()
  parser.add_argument("-u", "--username", help="Snapchat username ex. djkhaled305", type=str, required=True, dest="username")
  parser.add_argument("-o", "--output", help="Output video name ex. dj_khaled_stories", type=str, default="output", dest="output", metavar="OUTPUT_NAME")
  parser.add_argument("-d", "--delete", help="Delete stories after download.", action="store_true", default=False, dest="delete")
  parser.add_argument("--sleep-interval", help="Sleep between downloads in seconds. (Default: 1s)", type=int, default=1, dest="sleep_interval", metavar="INTERVAL")
  parser.add_argument("-l", "--limit-story", help="Set maximum number of stories to download.", type=int, default=-1, dest="limit_story", metavar="LIMIT")
  parser.add_argument("-v", "--verbose", help="FFmpeg output verbosity.", action="store_true", default=False, dest="verbose")
  parser.add_argument("--image-duration", help="Set duration for image in seconds. (Default: 1s)", type=int, default=1, dest="loop_duration_image", metavar="DURATION")

  if not is_ffmpeg_installed():
    logger.error("FFmpeg binary not found. Please install FFmpeg or add it to PATH.")
    logger.error("Download FFmpeg from https://ffmpeg.org/download.html")
    sys.exit(1)

  if len(sys.argv) == 1:
    parser.print_help()
    sys.exit(1)

  args = parser.parse_args()

  try:
    dir_name = SnapchatDL(sleep_interval=args.sleep_interval, limit_story=args.limit_story).download(args.username)
    concat_stories = ConcatStories(dir_name, args.output, loop_duration_image=args.loop_duration_image, is_quiet=not args.verbose)
    concat_stories.concat()

    if args.delete:
      shutil.rmtree(dir_name)
      logger.info(f"Stories deleted from {dir_name}")
  except Exception:
    sys.exit(1)


if __name__ == "__main__":
  sys.exit(main())



import ffmpeg
import os
from loguru import logger

RESOLUTION = (480, 852)
EXT = ".mp4"
FRAMERATE = 30


class ConcatStories:
  def __init__(self, dir_name: str, output: str, loop_duration_image: int = 1, is_quiet: bool = True):
    self.dir_name = dir_name
    stories = os.listdir(self.dir_name)
    stories.sort(key=lambda file: os.path.getmtime(os.path.join(self.dir_name, file)))
    self.stories = stories
    self.output = output
    self.resolution = RESOLUTION
    self.loop_duration_image = loop_duration_image
    self.is_quiet = is_quiet
    try:
      probe_file = list(filter(lambda file: file.endswith(EXT), stories))[0]
      probe_file = ffmpeg.probe(os.path.join(self.dir_name, probe_file))
      self.resolution = (probe_file['streams'][0]['width'], probe_file['streams'][0]['height'])
    except Exception as e:
      print(e)

  def concat(self):
    input_streams_spread = []

    for file in self.stories:
      if file.endswith(".mp4"):
        stream = ffmpeg.input(os.path.join(self.dir_name, file))
        probe = ffmpeg.probe(os.path.join(self.dir_name, file))
        if len(probe['streams']) < 2:
          empty_audio = ffmpeg.input(
              'anullsrc', f='lavfi', t=probe['streams'][0]['duration'])
          input_streams_spread.extend([stream, empty_audio])
        else:
          input_streams_spread.extend([stream['v'], stream['a']])
      else:
        image = ffmpeg.input(os.path.join(self.dir_name, file), t=self.loop_duration_image, loop=1, framerate=FRAMERATE)
        image = ffmpeg.filter_(image, 'scale', *self.resolution)
        empty_audio = ffmpeg.input('anullsrc', f='lavfi', t=self.loop_duration_image)
        input_streams_spread.extend([image, empty_audio])

    joined = ffmpeg.concat(*input_streams_spread, v=1, a=1, unsafe=True).node
    loglevel = "quiet" if self.is_quiet else "info"
    ffmpeg.output(joined[0], joined[1], os.path.join(".", self.output + ".mp4"), loglevel=loglevel).run()
    logger.info(f"Stories concatenated to {self.output}.mp4")
